"use strict";
// src/generate.ts
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ts_morph_1 = require("ts-morph");
const fs_1 = __importDefault(require("fs"));
// Utility function to generate hook names
function toHookName(tableName, operation) {
    return `use${tableName[0].toUpperCase() + tableName.slice(1)}${operation}`;
}
function generateHooks(typesPath, outputPath, supabaseClientPath) {
    console.log('Generating hooks with the following arguments:', {
        typesPath,
        outputPath,
        supabaseClientPath,
    });
    const project = new ts_morph_1.Project({
        compilerOptions: {
            allowSyntheticDefaultImports: true,
            esModuleInterop: true,
            module: ts_morph_1.ModuleKind.ESNext,
            target: ts_morph_1.ScriptTarget.ESNext,
        },
    });
    const sourceFile = project.addSourceFileAtPath(typesPath);
    // Find the 'Tables' type alias
    const databaseInterface = sourceFile.getInterfaceOrThrow('Database');
    const publicProperty = databaseInterface.getPropertyOrThrow('public');
    const publicType = publicProperty.getType();
    const tablesProperty = publicType
        .getApparentProperties()
        .find((property) => property.getName() === 'Tables');
    if (!tablesProperty) {
        throw new Error('No Tables property found within the Database interface.');
    }
    const tablesType = project
        .getProgram()
        .getTypeChecker()
        .getTypeAtLocation(tablesProperty.getValueDeclarationOrThrow());
    const tablesProperties = tablesType.getProperties();
    if (tablesProperties.length === 0) {
        throw new Error('No tables found within the Tables property.');
    }
    // Iterate through table keys and generate hooks
    const hooks = [];
    for (const table of tablesProperties) {
        const tableName = table.getName();
        // Generate hooks for fetching, adding, updating, and deleting
        hooks.push(`export function ${toHookName(tableName, 'ById')}(id: string) {
  return useQuery<Database['public']['Tables']['${tableName}']['Row'], Error>(
    ['${tableName}', id],
    async () => {
      const { data, error } = await supabase
        .from<Database['public']['Tables']['${tableName}']['Row']>('${tableName}')
        .select('*')
        .eq('id', id)
        .single();

      if (error) {
        throw error;
      }

      if (!data) {
        throw new Error('No data found');
      }

      return data;
    },
    {
      enabled: !!id,
    }
  );
}`, `export function ${toHookName(tableName, 'FetchAll')}() {
  return useQuery<Database['public']['Tables']['${tableName}']['Row'][], Error>(['${tableName}'], async () => {
    const { data, error } = await supabase.from<Database['public']['Tables']['${tableName}']['Row']>('${tableName}').select();
    if (error) throw error;
    return data as Database['public']['Tables']['${tableName}']['Row'][];
  });
}`, `export function ${toHookName(tableName, 'Add')}() {
  const queryClient = useQueryClient();
  return useMutation((item: Database['public']['Tables']['${tableName}']['Insert']) => supabase.from<Database['public']['Tables']['${tableName}']['Row']>('${tableName}').insert(item).single(), {
    onSuccess: () => {
      queryClient.invalidateQueries('${tableName}');
    },
  });
}`, `export function ${toHookName(tableName, 'Update')}() {
  const queryClient = useQueryClient();
  return useMutation((item: { id: string; changes: Database['public']['Tables']['${tableName}']['Update'] }) => supabase.from<Database['public']['Tables']['${tableName}']['Row']>('${tableName}').update(item.changes).eq('id', item.id).single(), {
    onSuccess: () => {
      queryClient.invalidateQueries('${tableName}');
    },
  });
}`, `export function ${toHookName(tableName, 'Delete')}() {
  const queryClient = useQueryClient();
  return useMutation((id: string) => supabase.from<Database['public']['Tables']['${tableName}']['Row']>('${tableName}').delete().eq('id', id).single(), {
    onSuccess: () => {
      queryClient.invalidateQueries('${tableName}');
    },
  });
}`);
    }
    // Create the output file content with imports and hooks
    const hooksFileContent = `
import { useMutation, useQuery, useQueryClient } from 'react-query';
import { Database } from '${typesPath}';
import { supabase } from '${supabaseClientPath}';

${hooks.join('\n\n')}
`;
    // Write the output file
    fs_1.default.writeFileSync(outputPath, hooksFileContent);
}
exports.default = generateHooks;
